package com.android.animateimage;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.SurfaceHolder.Callback;
import android.widget.TextView;

public class AnimateView extends SurfaceView implements Callback {

	class ImageThread extends Thread {
		private Bitmap mImgBack0;
		private Bitmap mImgBack1;
		private Bitmap mImgBack21;
		private Bitmap mImgBack31;
        private int mPlayerWidth;
        private int mPlayerHeight;
        private int mVisitCount = 0;
        private SurfaceHolder mSurfaceHolder;
        private Drawable mFrontImage[] = new Drawable[4];

        private Drawable mArrow[] = new Drawable[2];
        private int mArrowWidth, mArrowHeight;
        private int mDirection = 0;
        
        public ImageThread(SurfaceHolder surfaceHolder, Context context, Handler handler) {
            mSurfaceHolder = surfaceHolder;
            Resources res = context.getResources();

            mFrontImage[0] = context.getResources().getDrawable(
                    R.drawable.front01);
            mFrontImage[1] = context.getResources().getDrawable(
                    R.drawable.front02);
            mFrontImage[2] = context.getResources().getDrawable(
                    R.drawable.front03);
            mFrontImage[3] = context.getResources().getDrawable(
                    R.drawable.front04);

            mImgBack0 = BitmapFactory.decodeResource(res,
                    R.drawable.img0);
            mImgBack1 = BitmapFactory.decodeResource(res,
                    R.drawable.img1);
            mImgBack21 = BitmapFactory.decodeResource(res,
                    R.drawable.img21);
            mImgBack31 = BitmapFactory.decodeResource(res,
                    R.drawable.img31);

            // Use the regular lander image as the model size for all sprites
            mPlayerWidth = mFrontImage[1].getIntrinsicWidth();
            mPlayerHeight = mFrontImage[1].getIntrinsicHeight();

            mArrow[0] = context.getResources().getDrawable(
                    R.drawable.arrowleft);
            mArrow[1] = context.getResources().getDrawable(
                    R.drawable.arrowright);
            mArrowWidth = mArrow[0].getIntrinsicWidth();
            mArrowHeight = mArrow[0].getIntrinsicHeight();
        }

        public void doStart() {
            synchronized (mSurfaceHolder) {
            }
        }
        public void run() {
            while (true) {
                Canvas c = null;
                try {
                    c = mSurfaceHolder.lockCanvas(null);
                    synchronized (mSurfaceHolder) {
                        doDraw(c);
                        mVisitCount++;
                        sleep(100);
                    }
                } catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} finally {
                    // do this in a finally so that if an exception is thrown
                    // during the above, we don't leave the Surface in an
                    // inconsistent state
                    if (c != null) {
                        mSurfaceHolder.unlockCanvasAndPost(c);
                    }
                }
            }
        }

        private void doDraw(Canvas canvas) {
            // Draw the background image. Operations on the Canvas accumulate
            // so this is like clearing the screen.
            canvas.drawBitmap(mImgBack0, 0, 0, null);
            canvas.drawBitmap(mImgBack1, 0, 90, null);
            canvas.drawBitmap(mImgBack21, 0, 190, null);
            canvas.drawBitmap(mImgBack31, 0, 310, null);

            mFrontImage[mVisitCount%4].setBounds(145, 360, 145+mPlayerWidth, 360+mPlayerHeight);
            mFrontImage[mVisitCount%4].draw(canvas);

            if (mDirection == -1){
                mArrow[0].setBounds(20, 200, 20+mArrowWidth, 200+mArrowHeight);
                mArrow[0].draw(canvas);
            }
            else if (mDirection == 1){
                mArrow[1].setBounds(190, 200, 190+mArrowWidth, 200+mArrowHeight);
                mArrow[1].draw(canvas);
            }
            
        }
        
        void setKeyDown(int keyCode) {
        	mDirection = keyCode;
       	}      
	}
	
    private TextView mStatusText;
    private ImageThread thread;
	
	public AnimateView(Context context, AttributeSet attrs) {
		super(context, attrs);
		// TODO Auto-generated constructor stub
        // register our interest in hearing about changes to our surface
        SurfaceHolder holder = getHolder();
        holder.addCallback(this);

        // create thread only; it's started in surfaceCreated()
        thread = new ImageThread(holder, context, new Handler() {
            @Override
            public void handleMessage(Message m) {
                mStatusText.setVisibility(m.getData().getInt("viz"));
                mStatusText.setText(m.getData().getString("text"));
            }
        });

        setFocusable(true); // make sure we get key events
	}

	public void surfaceChanged(SurfaceHolder arg0, int arg1, int arg2, int arg3) {
		// TODO Auto-generated method stub

	}

	public void surfaceCreated(SurfaceHolder arg0) {
		// TODO Auto-generated method stub
		thread.start();
	}

	public void surfaceDestroyed(SurfaceHolder arg0) {
		// TODO Auto-generated method stub
        boolean retry = true;
        while (retry) {
            try {
                thread.join();
                retry = false;
            } catch (InterruptedException e) {
            }
        }
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		// TODO Auto-generated method stub
		if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT)
			thread.setKeyDown(-1);
		else if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) 
			thread.setKeyDown(1);
		else if (keyCode == KeyEvent.KEYCODE_DPAD_UP) 
			thread.setKeyDown(0);
			
		return super.onKeyDown(keyCode, event);
	}
}